@@ -37,7 +37,7 @@ int processtimeds(void);
 int processactions(struct timed *);
 void establishconnection(void);
 void reconnect(void);
-
+void proxysetup(void);
 void reconnect()
 {
 	char stringbuffer[WRITE_BUFFER_LENGTH + 1];
@@ -271,6 +271,7 @@ int serverconnection()
 		close(mystate->sockfd);
 		return OER_SERVERCONNECTION_ERR_GETSERVER_RETRY;
 	}
+        mystate->current_server->registerconnection_start = mystate->now;
 	mystate->current_server->used = 1;
 #ifdef DEBUG
 	oer_debug(OER_DEBUG_INFO, "serverconnection->using server %s port %d\n", mystate->current_server->serverhost, mystate->current_server->serverport);
@@ -313,6 +314,7 @@ int serverconnection()
 		close(mystate->sockfd);
 		return OER_SERVERCONNECTION_ERR_CONNECT;
 	}
+	proxysetup();
 	mystate->current_server->connected = 1;
         mystate->current_server->rx = 0;
         mystate->current_server->tx = 0;
@@ -342,6 +344,9 @@ int registerconnection()
 	   && mystate->current_server->registered.done) {
 		return OER_REGISTERCONNECTION_EVERYTHING_OK;
 	}
+	if((mystate->now - mystate->current_server->registerconnection_start) > OER_REGISTERCONNECTION_TIMEOUT) {
+		return OER_REGISTERCONNECTION_TIMEOUT;
+	}
 	if(!mystate->current_server->registered.begin) {
 		/* BEGIN */
 #ifdef DEBUG
@@ -601,6 +606,9 @@ int processactions(struct timed *t)
 	case OER_TIMED_PRIORITY_PRIVMSG:
 		mystate->last_action.barrier += 1 + (strlen(t->command) / 50);
 		break;
+	case OER_TIMED_PRIORITY_ADVERT:
+		mystate->last_action.barrier += 2 + (strlen(t->command) / 25);
+		break;
 	case OER_TIMED_PRIORITY_NORMAL:
 	case OER_TIMED_PRIORITY_CHANNEL_HANDLING:
 	case OER_TIMED_PRIORITY_CHANNEL_PROTECTION:
@@ -613,3 +621,22 @@ int processactions(struct timed *t)
 	}
 	return 1;
 }
+
+void proxysetup()
+{
+	char stringbuffer[STRINGLEN + 1];
+#ifdef DEBUG
+        oer_debug(OER_DEBUG_NOISE, "proxysetup()\n");
+#endif
+	if(strlen(mystate->proxysetup)) {
+		snprintf(stringbuffer, STRINGLEN, "%s\n", mystate->proxysetup);
+#ifdef DEBUG
+		oer_debug(OER_DEBUG_INFO, "proxysetup->%s", stringbuffer);
+#endif
+		if(write(mystate->sockfd, stringbuffer, strlen(stringbuffer)) < 0) {
+#ifdef DEBUG
+			oer_debug(OER_DEBUG_ERROR, "proxysetup->write failed: %s\n", strerror(errno));
+#endif
+		}
+	}
+}
