@@ -59,9 +59,11 @@ void parseirc(char *serveroutput)
 	char user[USERHOSTLEN + 1];
 	char host[USERHOSTLEN + 1];
 	char userhost[USERHOSTLEN + 1];
-	char nick[NICKLEN + 1];
-	char nick2[NICKLEN + 1];
+	/* NICKLEN is not sufficient since sometimes the source/target
+	   can be a service in which case user@host has to fit into the
+	   variable defined here */
 	char temp_nick[STRINGLEN + 1];
+	char temp_nick2[STRINGLEN + 1];
 	char target[STRINGLEN + 1];
 	char m;
 	char *restptr;
@@ -154,27 +156,27 @@ void parseirc(char *serveroutput)
 				break;
 			}
 			restpos = nrestpos;
-			memset(nick, 0, NICKLEN + 1);
+			memset(temp_nick, 0, STRINGLEN + 1);
 			tpos = 0;
-			if((tpos = parse(outstring, tpos, "=", nick, NICKLEN, 1)) < 0) {
+			if((tpos = parse(outstring, tpos, "=", temp_nick, STRINGLEN, 1)) < 0) {
 				return;
 			}
-			if(!strlen(nick)) {
+			if(!strlen(temp_nick)) {
 				/* USERHOST returns emptyline for invalid nicks */
 				return;
 			}
 			/* check for ircop, remove trailing star */
-			if(nick[strlen(nick) - 1] == '*') {
-				nick[strlen(nick) - 1] = '\0';
+			if(temp_nick[strlen(temp_nick) - 1] == '*') {
+				temp_nick[strlen(temp_nick) - 1] = '\0';
 				/* isoper */
 				for(this = mystate->channels; this != NULL; this = this->next) {
-					if(isonchan(this, nick)) {
-						changeuser(this, nick, 1, -1, -1, -1);
+					if(isonchan(this, temp_nick)) {
+						changeuser(this, temp_nick, 1, -1, -1, -1);
 					}
 				}
 			}
 			strncpy(userhost, outstring + tpos + 1, USERHOSTLEN);
-			if(isme(nick)) {
+			if(isme(temp_nick)) {
 				/* this is the forced USERHOST query from 001 */
 				strncpy(user, userhost, USERHOSTLEN);
 				ptr = index(user, '@');
@@ -205,9 +207,9 @@ void parseirc(char *serveroutput)
 				}
 			} else {
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_INFO, "parseirc(302)->nick %s has userhost %s\n", nick, userhost);
+				oer_debug(OER_DEBUG_INFO, "parseirc(302)->nick %s has userhost %s\n", temp_nick, userhost);
 #endif
-				setuserhost(nick, userhost);
+				setuserhost(temp_nick, userhost);
 			}
 		}
 		return;
@@ -231,6 +233,26 @@ void parseirc(char *serveroutput)
 		timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, timed_str);
 		return;
 	}
+	if(!strcasecmp(command, "331")) {
+		/* the channel has no topic set, returned to a TOPIC command
+		   without params (for a channel which has no topic) */
+		restpos = 0;
+		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
+			return;
+		}
+		if((this = getchptr(channel)) == NULL) {
+			return;
+		}
+                if(haschanflags(this, "T")) {
+                        /* skip topic handling for this channel */
+                        return;
+                }
+		/* replace the empty channel topic with ours, if we have one */
+		if(this->topics != NULL) {
+			this->topic_change = 1;
+		}
+		return;
+	}
 	if(!strcasecmp(command, "332")) {
 		restpos = 0;
 		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
@@ -279,7 +301,7 @@ void parseirc(char *serveroutput)
 		if((restpos = parse(restptr, restpos, " ", outstring, STRINGLEN, 0)) < 0) {
 			return;
 		}
-		if((restpos = parse(restptr, restpos, " ", nick, NICKLEN, 0)) < 0) {
+		if((restpos = parse(restptr, restpos, " ", temp_nick, STRINGLEN, 0)) < 0) {
 			return;
 		}
 		if((restpos = parse(restptr, restpos, " ", mode, CHANLEN, 0)) < 0) {
@@ -289,15 +311,15 @@ void parseirc(char *serveroutput)
 			return;
 		}
 		snprintf(userhost, USERHOSTLEN, "%s@%s", user, host);
-		setuserhost(nick, userhost);
+		setuserhost(temp_nick, userhost);
 		if(index(mode, (int)'@')) {
-			changeuser(this, nick, -1, 1, -1, -1);
+			changeuser(this, temp_nick, -1, 1, -1, -1);
 		}
 		if(index(mode, (int)'*')) {
-			changeuser(this, nick, 1, -1, -1, -1);
+			changeuser(this, temp_nick, 1, -1, -1, -1);
 		}
 		if(index(mode, (int)'+')) {
-			changeuser(this, nick, -1, -1, 1, -1);
+			changeuser(this, temp_nick, -1, -1, 1, -1);
 		}
 		return;
 	}
@@ -316,30 +338,30 @@ void parseirc(char *serveroutput)
 		oer_debug(OER_DEBUG_INFO, "parseirc(353)->%s\n", this->name);
 #endif
 		while(1) {
-			nrestpos = parse(restptr, restpos, " ", nick, NICKLEN, 0);
+			nrestpos = parse(restptr, restpos, " ", temp_nick, STRINGLEN, 0);
 			if(nrestpos == restpos || nrestpos < 0) {
 				break;
 			}
 			restpos = nrestpos;
-			if(nick[0] == ':') {
-				strncpy(nick2, nick + 1, NICKLEN);
-				strncpy(nick, nick2, NICKLEN);
+			if(temp_nick[0] == ':') {
+				strncpy(temp_nick2, temp_nick + 1, STRINGLEN);
+				strncpy(temp_nick, temp_nick2, STRINGLEN);
 			}
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(353)->%s\n", nick);
+			oer_debug(OER_DEBUG_INFO, "parseirc(353)->%s\n", temp_nick);
 #endif
 			m = '\0';
-			if(nick[0] == '*' || nick[0] == '@' || nick[0] == '+') {
-				m = nick[0];
-				strncpy(nick2, nick + 1, NICKLEN);
-				strncpy(nick, nick2, NICKLEN);
+			if(temp_nick[0] == '*' || temp_nick[0] == '@' || temp_nick[0] == '+') {
+				m = temp_nick[0];
+				strncpy(temp_nick2, temp_nick + 1, STRINGLEN);
+				strncpy(temp_nick, temp_nick2, STRINGLEN);
 			}
-			if(userjoined(this, nick, NULL, (m == '*') ? 1 : 0, (m == '@') ? 1 : 0, (m == '+') ? 1 : 0, 0) == NULL) {
+			if(userjoined(this, temp_nick, NULL, (m == '*') ? 1 : 0, (m == '@') ? 1 : 0, (m == '+') ? 1 : 0, 0) == NULL) {
 #ifdef DEBUG
 				oer_debug(OER_DEBUG_INFO, "parseirc(353)->userjoined() returned NULL\n");
 #endif
 			}
-			if(isme(nick) && m == '@') {
+			if(isme(temp_nick) && m == '@') {
 				this->i_am_op = 1;
 			}
 		}
@@ -434,7 +456,7 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "432") || !strcasecmp(command, "433")) {
 		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", nick, NICKLEN, 0)) < 0) {
+		if((restpos = parse(restptr, restpos, " ", temp_nick, STRINGLEN, 0)) < 0) {
 			return;
 		}
 		/* check if we are still registering */
@@ -448,7 +470,7 @@ void parseirc(char *serveroutput)
 				mystate->use_altnick = 0;
 			} else {
 #ifdef DEBUG
-				oer_debug(OER_DEBUG_INFO, "parseirc(%s)->our nickname %s is already in use, will try altnick %s\n", command, nick, mystate->altnick);
+				oer_debug(OER_DEBUG_INFO, "parseirc(%s)->our nickname %s is already in use, will try altnick %s\n", command, temp_nick, mystate->altnick);
 #endif
 				strncpy(mystate->nick, mystate->altnick, NICKLEN);
 				mystate->use_altnick = 1;
@@ -478,7 +500,7 @@ void parseirc(char *serveroutput)
 					mystate->use_altnick = 0;
 				} else {
 #ifdef DEBUG
-					oer_debug(OER_DEBUG_INFO, "parseirc(%s)->our nickname %s is already in use, will try altnick %s\n", command, nick, mystate->altnick);
+					oer_debug(OER_DEBUG_INFO, "parseirc(%s)->our nickname %s is already in use, will try altnick %s\n", command, channel, mystate->altnick);
 #endif
 					strncpy(mystate->nick, mystate->altnick, NICKLEN);
 					mystate->use_altnick = 1;
@@ -545,7 +567,7 @@ void parseirc(char *serveroutput)
 	if(!strcasecmp(command, "484")) {
 		/* undernet extension, protected user */
 		restpos = 0;
-		if((restpos = parse(restptr, restpos, " ", nick, NICKLEN, 0)) < 0) {
+		if((restpos = parse(restptr, restpos, " ", temp_nick, STRINGLEN, 0)) < 0) {
 			return;
 		}
 		if((restpos = parse(restptr, restpos, " ", channel, CHANLEN, 0)) < 0) {
@@ -555,9 +577,9 @@ void parseirc(char *serveroutput)
 			return;
 		}
 #ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "parseirc(484)->nick %s on %s is a IRC operator\n", nick, this->name);
+		oer_debug(OER_DEBUG_INFO, "parseirc(484)->nick %s on %s is a IRC operator\n", temp_nick, this->name);
 #endif
-		changeuser(this, nick, 1, -1, -1, -1);
+		changeuser(this, temp_nick, 1, -1, -1, -1);
 		return;
 	}
 	if(!strcasecmp(command, "513")) {
@@ -579,22 +601,22 @@ void parseirc(char *serveroutput)
 			return;
 		}
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
+		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
-		if(isme(nick)) {
+		if(isme(temp_nick)) {
 			/* we are not interested in our own joins */
 			return;
 		}
 		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
-		if(userjoined(this, nick, userhost, 0, 0, 0, isfriend(this, nick, userhost)) == NULL) {
+		if(userjoined(this, temp_nick, userhost, 0, 0, 0, isfriend(this, temp_nick, userhost)) == NULL) {
 #ifdef DEBUG
 			oer_debug(OER_DEBUG_INFO, "parseirc(JOIN)->userjoined() returned NULL\n");
 #endif
 		}
-		addnewjoin(this, mystate->now, nick, userhost);
+		addnewjoin(this, mystate->now, temp_nick, userhost);
 		if(haschanflags(this, "S")) {
-			updateseen(this, nick, userhost);
+			updateseen(this, temp_nick, userhost);
 		}
 		if(mystate->netjoining) {
 			return;
@@ -610,35 +632,35 @@ void parseirc(char *serveroutput)
 		listchanusers();
 		/* the on-join checks, they require that oer is opped */
 		if(this->i_am_op) {
-			if(!gettobek(this, nick) && checkforautorejoin(this, nick, userhost)) {
-				changetobek(this, nick, 1);
+			if(!gettobek(this, temp_nick) && checkforautorejoin(this, temp_nick, userhost)) {
+				changetobek(this, temp_nick, 1);
 			}
-			if(!gettobek(this, nick) && checkforpartrejoin(this, nick, userhost)) {
-				changetobek(this, nick, 1);
-				resetparts(this, nick, userhost);
+			if(!gettobek(this, temp_nick) && checkforpartrejoin(this, temp_nick, userhost)) {
+				changetobek(this, temp_nick, 1);
+				resetparts(this, temp_nick, userhost);
 			}
-			if(!gettobek(this, nick) && checkforbadnick(this, nick, userhost)) {
-				changetobek(this, nick, 1);
+			if(!gettobek(this, temp_nick) && checkforbadnick(this, temp_nick, userhost)) {
+				changetobek(this, temp_nick, 1);
 			}
-			if(!gettobek(this, nick) && haschanflags(this, "V") && isopa(userhost)) {
-				mmode_new(this, mystate->now, "+v", nick);
+			if(!gettobek(this, temp_nick) && haschanflags(this, "V") && isopa(userhost)) {
+				mmode_new(this, mystate->now, "+v", temp_nick);
 			}
-			if(!gettobek(this, nick) && haschanflags(this, "v") && isvoice(this, userhost)) {
-				mmode_new(this, mystate->now, "+v", nick);
+			if(!gettobek(this, temp_nick) && haschanflags(this, "v") && isvoice(this, userhost)) {
+				mmode_new(this, mystate->now, "+v", temp_nick);
 			}
-			if(!gettobek(this, nick) && haschanflags(this, "O") && isopa(userhost)) {
-				mmode_new(this, mystate->now, "+o", nick);
+			if(!gettobek(this, temp_nick) && haschanflags(this, "O") && isopa(userhost)) {
+				mmode_new(this, mystate->now, "+o", temp_nick);
 			}
-			if(!gettobek(this, nick) && haschanflags(this, "o") && isop(this, userhost)) {
-				mmode_new(this, mystate->now, "+o", nick);
+			if(!gettobek(this, temp_nick) && haschanflags(this, "o") && isop(this, userhost)) {
+				mmode_new(this, mystate->now, "+o", temp_nick);
 			}
 		}
 		/* if the user will be kicked, skip the rest */
-		if(gettobek(this, nick)) {
+		if(gettobek(this, temp_nick)) {
 			return;
 		}
 		if(haschanflags(this, "A")) {
-			sendadverts(this, nick, userhost);
+			sendadverts(this, temp_nick, userhost);
 		}
 		/* then possibly the on-join quote */
 		if(!haschanflags(this, "q")) {
@@ -650,22 +672,22 @@ void parseirc(char *serveroutput)
 		if((mystate->now - this->last_quote) < OER_QUOTE_INTERVAL) {
 			return;
 		}
-		if(getjoincount(this, nick) > 1) {
+		if(getjoincount(this, temp_nick) > 1) {
 			return;
 		}
-		if((msgcount = getondiskmsgcount(this, nick)) <= 0) {
+		if((msgcount = getondiskmsgcount(this, temp_nick)) <= 0) {
 			return;
 		}
-		strncpy(nick2, nick, NICKLEN);
+		strncpy(temp_nick2, temp_nick, STRINGLEN);
 		gettimeofday(&tv1, NULL);
-		if(getrandommsg(this, nick2, NICKLEN, getrandom(msgcount), outstring2, HUGESTRINGLEN) < 0) {
+		if(getrandommsg(this, temp_nick2, STRINGLEN, getrandom(msgcount), outstring2, HUGESTRINGLEN) < 0) {
 			return;
 		}
 		if(!isgoodquote(outstring2)) {
 			return;
 		}
 		gettimeofday(&tv2, NULL);
-		snprintf(outstring3, HUGESTRINGLEN, "\"%s\" (c) %s", outstring2, nick2);
+		snprintf(outstring3, HUGESTRINGLEN, "\"%s\" (c) %s", outstring2, temp_nick2);
 		sendreply(channel, 1, 0, OER_TIMED_PRIORITY_NORMAL, outstring3);
 		this->last_quote = mystate->now;
 		snprintf(outstring3, HUGESTRINGLEN, "%ld secs and %ld usecs", tv2.tv_sec - tv1.tv_sec, ABS(tv2.tv_usec - tv1.tv_usec));
@@ -684,28 +706,28 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "PRIVMSG")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
+		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
 		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
 		strncpy(channel, params, CHANLEN);
 		/* the message starts at pos + 1 ... */
-		parsemsg(nick, userhost, channel, restptr + 1);
+		parsemsg(temp_nick, userhost, channel, restptr + 1);
 		return;
 	}
 	if(!strcasecmp(command, "QUIT")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
+		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
 		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
 		/* quit doesn't provide the channel, we have to check all channels */
 		for(this = mystate->channels; this != NULL; this = this->next) {
-			if(!isonchan(this, nick)) {
+			if(!isonchan(this, temp_nick)) {
 				continue;
 			}
-			addnewpart(this, mystate->now, nick, userhost);
-			if(!userleft(this, nick, userhost)) {
+			addnewpart(this, mystate->now, temp_nick, userhost);
+			if(!userleft(this, temp_nick, userhost)) {
 #ifdef DEBUG
 				oer_debug(OER_DEBUG_INFO, "parseirc(QUIT)->userleft() returned NULL\n");
 #endif
@@ -723,7 +745,7 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "PART")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
+		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
 		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
@@ -731,7 +753,7 @@ void parseirc(char *serveroutput)
 		if((this = getchptr(channel)) == NULL) {
 			return;
 		}
-		if(isme(nick)) {
+		if(isme(temp_nick)) {
 			this->rejoin_at = mystate->now + OER_DELAY_BETWEEN_REJOINS;
 			if(this->nickcount == 1 && !this->i_am_op) {
 				/* we are going to cycle */
@@ -741,8 +763,8 @@ void parseirc(char *serveroutput)
 			initchannel(this);
 			return;
 		}
-		addnewpart(this, mystate->now, nick, userhost);
-		if(!userleft(this, nick, userhost)) {
+		addnewpart(this, mystate->now, temp_nick, userhost);
+		if(!userleft(this, temp_nick, userhost)) {
 #ifdef DEBUG
 			oer_debug(OER_DEBUG_INFO, "parseirc(PART)->userleft() returned NULL\n");
 #endif
@@ -759,42 +781,41 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "NICK")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
+		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
 		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
-		if(isme(nick)) {
+		if(isme(temp_nick)) {
 			/* oer changed it's nick either because it was 
 			 asked to or it wasn't the primary nick and
 			global flag "g" was set */
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "nickchange->I changed my nick from %s to %s\n", nick, params + 1);
+			oer_debug(OER_DEBUG_INFO, "nickchange->I changed my nick from %s to %s\n", temp_nick, params + 1);
 #endif
 			strncpy(mystate->nick, params + 1, NICKLEN);
-			return;
 		}
 #ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "nickchange->%s is now known as %s, changing on all channels\n", nick, params + 1);
+		oer_debug(OER_DEBUG_INFO, "nickchange->%s is now known as %s, changing on all channels\n", temp_nick, params + 1);
 #endif
 		/* if user changes nick while we are doing HOSTNAME queries
 		   on channel join, we will end up with userhost = NULL for the user */
-		if(ishostless(nick) == 1) {
-			sethostquerystatus(nick, 0);
+		if(ishostless(temp_nick) == 1) {
+			sethostquerystatus(temp_nick, 0);
 		}
 		/* nick doesn't provide the channel, we have to check all channels */
 		for(this = mystate->channels; this != NULL; this = this->next) {
-			if(isonchan(this, nick)) {
+			if(isonchan(this, temp_nick)) {
 				addnewnickchange(this, userhost);
 				/* if a user tries to avoid getting kicked
 				   on !lock by changing nick, we will adapt */
-				changetobek(this, nick, 0);
+				changetobek(this, temp_nick, 0);
 				if(this->i_am_op && checkforbadnick(this, params + 1, userhost)) {
-					changetobek(this, nick, 1);
+					changetobek(this, temp_nick, 1);
 				}
-				if(this->i_am_op && checkfornickflood(this, nick, userhost)) {
-					changetobek(this, nick, 1);
+				if(this->i_am_op && checkfornickflood(this, temp_nick, userhost)) {
+					changetobek(this, temp_nick, 1);
 				}
-				nickchange(this, nick, params + 1);
+				nickchange(this, temp_nick, params + 1);
 			}
 		}
 		return;
@@ -809,7 +830,7 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "KICK")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
+		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
 		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
@@ -823,7 +844,7 @@ void parseirc(char *serveroutput)
 		}
 		if(isme(target)) {
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->I was kicked off %s by %s!%s\n", channel, nick, userhost);
+			oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->I was kicked off %s by %s!%s\n", channel, temp_nick, userhost);
 #endif
 			initchannel(this);
 			this->rejoin_at = mystate->now + OER_DELAY_BETWEEN_REJOINS;
@@ -836,7 +857,7 @@ void parseirc(char *serveroutput)
                         return;
                 }
 #ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->%s!%s off %s by %s!%s\n", target, (cu->userhost == NULL) ? "(null)" : cu->userhost, channel, nick, userhost);
+		oer_debug(OER_DEBUG_INFO, "parseirc(KICK)->%s!%s off %s by %s!%s\n", target, (cu->userhost == NULL) ? "(null)" : cu->userhost, channel, temp_nick, userhost);
 #endif
 		addnewpart(this, mystate->now, target, cu->userhost);
 		if(!userleft(this, target, cu->userhost)) {
@@ -858,7 +879,6 @@ void parseirc(char *serveroutput)
 		return;
 	}
 	if(!strcasecmp(command, "MODE")) {
-                /* NOTICE! Nick can also be a host (server/service modes) */
 		tpos = 0;
 		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
@@ -1087,7 +1107,7 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "TOPIC")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
+		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
 		strncpy(channel, params, CHANLEN);
@@ -1100,7 +1120,7 @@ void parseirc(char *serveroutput)
                 }
                 if(haschanflags(this, "t")) {
 			/* only allow topic changes through us */
-			if(!isme(nick)) {
+			if(!isme(temp_nick)) {
 				this->topic_change = 1;
 			}
 		}
@@ -1108,7 +1128,7 @@ void parseirc(char *serveroutput)
 	}
 	if(!strcasecmp(command, "INVITE")) {
 		tpos = 0;
-		if((tpos = parse(prefix + 1, tpos, "!", nick, NICKLEN, 1)) < 0) {
+		if((tpos = parse(prefix + 1, tpos, "!", temp_nick, STRINGLEN, 1)) < 0) {
 			return;
 		}
 		strncpy(userhost, prefix + 1 + tpos, USERHOSTLEN);
@@ -1117,7 +1137,7 @@ void parseirc(char *serveroutput)
 			return;
 		}
 #ifdef DEBUG
-		oer_debug(OER_DEBUG_INFO, "parseirc(INVITE)->%s invited me to %s\n", nick, channel + 1);
+		oer_debug(OER_DEBUG_INFO, "parseirc(INVITE)->%s invited me to %s\n", temp_nick, channel + 1);
 #endif
 		return;
 	}
@@ -1188,7 +1208,7 @@ void parsemsg(char *nick, char *userhost, char *target, char *message)
 		return;
 	}
 	if(isme(target)) {
-		/* the message is sent to me, not to a channel */
+		/* the message is sent to me and not to a channel */
 		parsecommand(target, nick, userhost, message);
 		return;
 	}
@@ -1229,8 +1249,8 @@ void parsemsg(char *nick, char *userhost, char *target, char *message)
 		}
 	}
 	if(is_command) {
-		/* here is when we parse all commands */
-		parsecommand(target, nick, userhost, message + ppos);
+		/* the message is sent to a channel -> parse */
+		parsecommand(this->name, nick, userhost, message + ppos);
 	}
 	/* it is possible that the channel was deleted */
 	if((this = getchptr(target)) == NULL) {
@@ -1257,6 +1277,7 @@ void parsemsg(char *nick, char *userhost, char *target, char *message)
 		updatelast(this, nick, userhost, message);
 	}
 }
+
 void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 {
 	int proceed;
@@ -1995,6 +2016,9 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->INVITE\n");
 #endif
+		if(tochan) {
+			return;
+		}
 		if((this = getchptr(channel)) == NULL) {
 			return;
 		}
@@ -2007,10 +2031,6 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
                 if(!isatleastopnow(this, nick, userhost)) {
                         return;
                 }
-		if(tochan) {
-			/* this makes no sense */
-			return;
-		}
 		snprintf(timed_str, WRITE_BUFFER_LENGTH, "INVITE %s %s", nick, this->name);
 		timed_new(NULL, mystate->now, OER_TIMED_TYPE_NORMAL, OER_TIMED_PRIORITY_CHANNEL_HANDLING, timed_str);
 		return;
@@ -2074,21 +2094,31 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->HELP\n");
 #endif
-		/* may be NULL */
-		this = getchptr(channel);
+		/* channel is optional */
 		proceed = (isopa(userhost)) ? 1 : 0;
-		for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
-			if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
-				continue;
+		if(tochan) {
+			if((this = getchptr(channel)) == NULL) {
+				return;
 			}
-			if(isatleastopnow(that, nick, userhost)) {
+			if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
+				return;
+			}
+			if(isatleastopnow(this, nick, userhost)) {
 				proceed = 1;
 			}
-		}
-		/* check if message was sent to a channel and if it was
-		   is "s" flag set */
-		if(tochan && haschanflags(this, "s")) {
-			proceed = 1;
+			/* check if "s" flag set */
+			if(haschanflags(this, "s")) {
+				proceed = 1;
+			}
+		} else {
+			for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
+				if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
+					continue;
+				}
+				if(isatleastopnow(that, nick, userhost)) {
+					proceed = 1;
+				}
+			}
 		}
 		if(!proceed) {
 			return;
@@ -2100,20 +2130,30 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->LIST\n");
 #endif
-		/* may be NULL */
+		if(!strlen(restptr)) {
+			return;
+		}
+		/* channel is optional */
+		proceed = (isopa(userhost)) ? 1 : 0;
 		this = getchptr(channel);
-                proceed = (isopa(userhost)) ? 1 : 0;
-		/* for some lists it is sufficient that the user
-		   is on one of our channels */
-		for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
-			if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
-				continue;
+		if(tochan) {
+			if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
+				return;
 			}
-			if(isatleastopnow(that, nick, userhost)) {
+			if(isatleastopnow(this, nick, userhost)) {
 				proceed = 1;
 			}
+		} else {
+			for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
+				if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
+					continue;
+				}
+				if(isatleastopnow(that, nick, userhost)) {
+					proceed = 1;
+				}
+			}
 		}
-		if(!proceed || !strlen(restptr)) {
+		if(!proceed) {
 			return;
 		}
 		listcommand(this, (tochan) ? this->name : nick, tochan, nick, userhost, restptr);
@@ -2122,51 +2162,96 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->ADD\n");
 #endif
-		/* may be NULL */
-		this = getchptr(channel);
-		if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
-                        return;
-                }
-                if(!isatleastopnow(this, nick, userhost)) {
-                        return;
-                }
                 if(!strlen(restptr)) {
                         return;
                 }
+		/* channel is optional */
+		proceed = (isopa(userhost)) ? 1 : 0;
+		this = getchptr(channel);
+		if(tochan) {
+			if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
+				return;
+			}
+			if(isatleastopnow(this, nick, userhost)) {
+				proceed = 1;
+			}
+		} else {
+			for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
+				if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
+					continue;
+				}
+				if(isatleastopnow(that, nick, userhost)) {
+					proceed = 1;
+				}
+			}
+		}
+		if(!proceed) {
+			return;
+		}
 		addcommand(this, (tochan) ? this->name : nick, tochan, nick, userhost, restptr);
 		return;
 	case OER_WHICHCOMMAND_DEL:
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->DEL\n");
 #endif
-		/* may be NULL */
-		this = getchptr(channel);
-		if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
-                        return;
-                }
-                if(!isatleastopnow(this, nick, userhost)) {
-                        return;
-                }
                 if(!strlen(restptr)) {
                         return;
                 }
+		/* channel is optional */
+		proceed = (isopa(userhost)) ? 1 : 0;
+		this = getchptr(channel);
+		if(tochan) {
+			if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
+				return;
+			}
+			if(isatleastopnow(this, nick, userhost)) {
+				proceed = 1;
+			}
+		} else {
+			for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
+				if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
+					continue;
+				}
+				if(isatleastopnow(that, nick, userhost)) {
+					proceed = 1;
+				}
+			}
+		}
+		if(!proceed) {
+			return;
+		}
 		delcommand(this, (tochan) ? this->name : nick, tochan, nick, userhost, restptr);
 		return;
 	case OER_WHICHCOMMAND_EDIT:
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->EDIT\n");
 #endif
-		/* may be NULL */
-		this = getchptr(channel);
-		if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
-                        return;
-                }
-                if(!isatleastopnow(this, nick, userhost)) {
-                        return;
-                }
                 if(!strlen(restptr)) {
                         return;
                 }
+		/* channel is optional */
+		proceed = (isopa(userhost)) ? 1 : 0;
+		this = getchptr(channel);
+		if(tochan) {
+			if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
+				return;
+			}
+			if(isatleastopnow(this, nick, userhost)) {
+				proceed = 1;
+			}
+		} else {
+			for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
+				if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
+					continue;
+				}
+				if(isatleastopnow(that, nick, userhost)) {
+					proceed = 1;
+				}
+			}
+		}
+		if(!proceed) {
+			return;
+		}
 		editcommand(this, (tochan) ? this->name : nick, tochan, nick, userhost, restptr);
 		return;
 	case OER_WHICHCOMMAND_LOGOFF:
@@ -2224,21 +2309,31 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->INFO\n");
 #endif
-		/* may be NULL */
-		this = getchptr(channel);
-                proceed = (isopa(userhost)) ? 1 : 0;
-		for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
-			if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
-				continue;
+		/* channel is optional */
+		proceed = (isopa(userhost)) ? 1 : 0;
+		if(tochan) {
+			if((this = getchptr(channel)) == NULL) {
+				return;
 			}
-			if(isatleastopnow(that, nick, userhost)) {
+			if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
+				return;
+			}
+			if(isatleastopnow(this, nick, userhost)) {
 				proceed = 1;
 			}
-		}
-		/* check if message was sent to a channel and if it was
-		   is "s" flag set */
-		if(tochan && haschanflags(this, "s")) {
-			proceed = 1;
+			/* check if "s" flag set */
+			if(haschanflags(this, "s")) {
+				proceed = 1;
+			}
+		} else {
+			for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
+				if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
+					continue;
+				}
+				if(isatleastopnow(that, nick, userhost)) {
+					proceed = 1;
+				}
+			}
 		}
 		if(!proceed) {
 			return;
@@ -2250,21 +2345,31 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
 		oer_debug(OER_DEBUG_INFO, "parsecommand->UPTIME\n");
 #endif
-		/* may be NULL */
-		this = getchptr(channel);
-                proceed = (isopa(userhost)) ? 1 : 0;
-		for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
-			if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
-				continue;
+		/* channel is optional */
+		proceed = (isopa(userhost)) ? 1 : 0;
+		if(tochan) {
+			if((this = getchptr(channel)) == NULL) {
+				return;
 			}
-			if(isatleastopnow(that, nick, userhost)) {
+			if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
+				return;
+			}
+			if(isatleastopnow(this, nick, userhost)) {
 				proceed = 1;
 			}
-		}
-		/* check if message was sent to a channel and if it was
-		   is "s" flag set */
-		if(tochan && haschanflags(this, "s")) {
-			proceed = 1;
+			/* check if "s" flag set */
+			if(haschanflags(this, "s")) {
+				proceed = 1;
+			}
+		} else {
+			for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
+				if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
+					continue;
+				}
+				if(isatleastopnow(that, nick, userhost)) {
+					proceed = 1;
+				}
+			}
 		}
 		if(!proceed) {
 			return;
@@ -2496,25 +2601,35 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
                 oer_debug(OER_DEBUG_INFO, "parsecommand->SSTATS\n");
 #endif
-		/* may be NULL */
-		this = getchptr(channel);
-                proceed = (isopa(userhost)) ? 1 : 0;
-                for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
-                        if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
-                                continue;
-                        }
-                        if(isatleastopnow(that, nick, userhost)) {
-                                proceed = 1;
-                        }
-                }
-		/* check if message was sent to a channel and if it was
-		   is "s" flag set */
-		if(tochan && haschanflags(this, "s")) {
-			proceed = 1;
+		/* channel is optional */
+		proceed = (isopa(userhost)) ? 1 : 0;
+		if(tochan) {
+			if((this = getchptr(channel)) == NULL) {
+				return;
+			}
+			if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
+				return;
+			}
+			if(isatleastopnow(this, nick, userhost)) {
+				proceed = 1;
+			}
+			/* check if "s" flag set */
+			if(haschanflags(this, "s")) {
+				proceed = 1;
+			}
+		} else {
+			for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
+				if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
+					continue;
+				}
+				if(isatleastopnow(that, nick, userhost)) {
+					proceed = 1;
+				}
+			}
+		}
+		if(!proceed) {
+			return;
 		}
-                if(!proceed) {
-                        return;
-                }
                 secondstostring(mystate->now - mystate->current_server->linkup, uptimestring, STRINGLEN);
                 snprintf(outstring, WRITE_BUFFER_LENGTH, "%s RX/TX bytes: %d/%d, link connected for %s", mystate->current_server->serverhost, mystate->current_server->rx, mystate->current_server->tx, uptimestring);
                 sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
@@ -2523,25 +2638,35 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
                 oer_debug(OER_DEBUG_INFO, "parsecommand->DATE\n");
 #endif
-		/* may be NULL */
-		this = getchptr(channel);
-                proceed = (isopa(userhost)) ? 1 : 0;
-                for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
-                        if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
-                                continue;
-                        }
-                        if(isatleastopnow(that, nick, userhost)) {
-                                proceed = 1;
-                        }
-                }
-		/* check if message was sent to a channel and if it was
-		   is "s" flag set */
-		if(tochan && haschanflags(this, "s")) {
-			proceed = 1;
+		/* channel is optional */
+		proceed = (isopa(userhost)) ? 1 : 0;
+		if(tochan) {
+			if((this = getchptr(channel)) == NULL) {
+				return;
+			}
+			if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
+				return;
+			}
+			if(isatleastopnow(this, nick, userhost)) {
+				proceed = 1;
+			}
+			/* check if "s" flag set */
+			if(haschanflags(this, "s")) {
+				proceed = 1;
+			}
+		} else {
+			for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
+				if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
+					continue;
+				}
+				if(isatleastopnow(that, nick, userhost)) {
+					proceed = 1;
+				}
+			}
+		}
+		if(!proceed) {
+			return;
 		}
-                if(!proceed) {
-                        return;
-                }
 		strncpy(ts, ctime(&mystate->now), TINYSTRINGLEN);
                 striplf(ts);
                 tzset();
@@ -2623,9 +2748,35 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 #ifdef DEBUG
                 oer_debug(OER_DEBUG_INFO, "parsecommand->BSTATS\n");
 #endif
-		/* may be NULL */
-		this = getchptr(channel);
-                proceed = (isopa(userhost)) ? 1 : 0;
+		/* channel is optional */
+		proceed = (isopa(userhost)) ? 1 : 0;
+		if(tochan) {
+			if((this = getchptr(channel)) == NULL) {
+				return;
+			}
+			if(haschanflags(this, "U") && !isatleastop(this, nick, userhost)) {
+				return;
+			}
+			if(isatleastopnow(this, nick, userhost)) {
+				proceed = 1;
+			}
+			/* check if "s" flag set */
+			if(haschanflags(this, "s")) {
+				proceed = 1;
+			}
+		} else {
+			for(that = mystate->channels; that != NULL && !proceed; that = that->next) {
+				if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
+					continue;
+				}
+				if(isatleastopnow(that, nick, userhost)) {
+					proceed = 1;
+				}
+			}
+		}
+		if(!proceed) {
+			return;
+		}
 		numofchans = 0;
 		numofadmins = 0;
 		numofusers = 0;
@@ -2650,20 +2801,6 @@ void parsecommand(char *target, char *nick, char *userhost, char *commandline)
 					numofusermasks++;
 				}
 			}
-			if(haschanflags(that, "U") && !isatleastop(that, nick, userhost)) {
-				continue;
-			}
-			if(isatleastopnow(that, nick, userhost)) {
-				proceed = 1;
-                        }
-                }
-		/* check if message was sent to a channel and if it was
-		   is "s" flag set */
-		if(tochan && haschanflags(this, "s")) {
-			proceed = 1;
-		}
-                if(!proceed) {
-                        return;
                 }
                 snprintf(outstring, WRITE_BUFFER_LENGTH, "%d %s (%d %s), %d %s (%d %s), %d channel %s and %d %s", numofadmins, (numofadmins == 1) ? "admin" : "admins", numofadminmasks, (numofadminmasks == 1) ? "mask" : "masks", numofusers, (numofusers == 1) ? "user" : "users", numofusermasks, (numofusermasks == 1) ? "mask" : "masks", numofchanusers, (numofchanusers == 1) ? "user" : "users", numofchans, (numofchans == 1) ? "channel" : "channels");
 		sendreply((tochan) ? channel : nick, tochan, 0, OER_TIMED_PRIORITY_NORMAL, outstring);
