@@ -41,15 +41,12 @@ do
 done
 
 $ECHO "Install tool is $INSTALL (params: $PARAMS)"
-$ECHO ""
-readln "Install oer binary into [$INSTALLTO] " "$INSTALLTO"
-INSTALLTO=$ans
+$ECHO "Installing oer binary into $INSTALLTO (change with configure)"
 if [ ! -d $INSTALLTO ]; then
-    $ECHO "Creating directory $INSTALLTO"
-
+    $ECHO "  Creating directory $INSTALLTO"
     mkdir -p $INSTALLTO
 fi
-$ECHO "Installing oer binary into directory $INSTALLTO"
+$ECHO "  Installing oer binary into directory $INSTALLTO"
 $INSTALL $PARAMS -m 755 ./oer $INSTALLTO/oer
 
 DEFANS1="Y"
@@ -83,6 +80,8 @@ if [ "$CREATECONF" = "Y" -o "$CREATECONF" = "y" ]; then
     REALNAME=$ans
     readln "Please specify optional virtual host [] " ""
     VIRTHOST=$ans
+    readln "Please specify optional proxy setup string [] " ""
+    PROXYSETUP=$ans
     readln "Please specify default IRC server to connect to [irc1.equnet.org] " "irc1.equnet.org"
     SERVER=$ans
     readln "Please specify port [6667] " "6667"
@@ -110,6 +109,9 @@ if [ "$CREATECONF" = "Y" -o "$CREATECONF" = "y" ]; then
     if [ x$VIRTHOST != x ]; then
 	$ECHO "vhost::$VIRTHOST" >> configuration/oer.conf
     fi
+    if [ x$PROXYSETUP != x ]; then
+	$ECHO "proxysetup::$PROXYSETUP" >> configuration/oer.conf
+    fi
     $ECHO "server::$SERVER::$PORT::$MODES::$PINGFREQ::$PROT::$NOISE::$SERVERPW" >> configuration/oer.conf
     
     RC=$?
